@@ -3,6 +3,7 @@
 #include <stdint.h>
 #include "simd_abpoa_align.h"
 #include "abpoa_align.h"
+#include "abpoa_seq.h"
 #include "utils.h"
 
 void gen_simple_mat(int m, int *mat, int match, int mismatch) {
@@ -44,6 +45,7 @@ abpoa_para_t *abpoa_init_para(void) {
     abpt->min_freq = DIPLOID_MIN_FREQ; 
     abpt->cons_agrm = ABPOA_HB;   // consensus calling algorithm 
     abpt->use_read_ids = 0;
+    abpt->incr_fn = NULL; // incrementally align seq to an existing graph
     abpt->out_pog = NULL; // dump partial order graph to file
 
     // number of residue types
@@ -77,6 +79,7 @@ void abpoa_post_set_para(abpoa_para_t *abpt) {
 void abpoa_free_para(abpoa_para_t *abpt) {
     if (abpt->mat != NULL) free(abpt->mat);
     if (abpt->out_pog != NULL) free(abpt->out_pog);
+    if (abpt->incr_fn != NULL) free(abpt->incr_fn);
     free(abpt);
 }
 
@@ -107,30 +110,47 @@ int abpoa_align_sequence_to_graph(abpoa_t *ab, abpoa_para_t *abpt, uint8_t *quer
 //    seqs: array of input sequences, 0123 for ACGT, size: seq_n * seq_len[]
 int abpoa_msa(abpoa_t *ab, abpoa_para_t *abpt, int n_seqs, char **seq_names, int *seq_lens, uint8_t **seqs, FILE *out_fp, uint8_t ***cons_seq, int ***cons_cov, int **cons_l, int *cons_n, uint8_t ***msa_seq, int *msa_l) {
     if ((!abpt->out_msa && !abpt->out_cons && !abpt->out_gfa) || n_seqs <= 0) return 0;
-    int i, tot_n = n_seqs;
-    uint8_t *is_rc = (uint8_t*)_err_malloc(n_seqs * sizeof(uint8_t));
+    int i;
+    // set ab->abs, name
+    abpoa_seq_t *abs = ab->abs;
+    abs->n_seq = n_seqs;
+    if (n_seqs > abs->m_seq) {
+        abs->m_seq = n_seqs;;
+        abs->name = (abpoa_str_t*)_err_realloc(abs->name, abs->m_seq * sizeof(abpoa_str_t));
+        abs->is_rc = (uint8_t*)_err_realloc(abs->is_rc, abs->m_seq * sizeof(uint8_t));
+    }
+    if (seq_names) {
+        for (i = 0; i < n_seqs; ++i) {
+            int l = strlen(seq_names[i]);
+            abpoa_cpy_str(abs->name+i, seq_names[i], l);
+        }
+    } else {
+        for (i = 0; i < n_seqs; ++i) {
+            abs->name[i].l = 0; abs->name[i].m = 0;
+        }
+    }
+
     abpoa_reset_graph(ab, abpt, seq_lens[0]);
     for (i = 0; i < n_seqs; ++i) {
         abpoa_res_t res;
         res.graph_cigar = 0, res.n_cigar = 0, res.is_rc = 0;
         abpoa_align_sequence_to_graph(ab, abpt, seqs[i], seq_lens[i], &res);
         abpoa_add_graph_alignment(ab, abpt, seqs[i], seq_lens[i], res, i, n_seqs);
-        is_rc[i] = res.is_rc;
+        abs->is_rc[i] = res.is_rc;
         if (res.n_cigar) free(res.graph_cigar);
     }
     if (abpt->out_gfa) {
-        abpoa_generate_gfa(ab, abpt, seq_names, is_rc, n_seqs, out_fp);
+        abpoa_generate_gfa(ab, abpt, out_fp);
     } else {
         if (abpt->out_cons) {
-            if (abpt->out_msa) abpoa_generate_consensus(ab, abpt, tot_n, NULL, cons_seq, cons_cov, cons_l, cons_n);
-            else abpoa_generate_consensus(ab, abpt, tot_n, out_fp, cons_seq, cons_cov, cons_l, cons_n);
+            if (abpt->out_msa) abpoa_generate_consensus(ab, abpt, NULL, cons_seq, cons_cov, cons_l, cons_n);
+            else abpoa_generate_consensus(ab, abpt, out_fp, cons_seq, cons_cov, cons_l, cons_n);
             if (ab->abg->is_called_cons == 0)
                 err_printf("Warning: no consensus sequence generated.\n");
         }
         if (abpt->out_msa) {
-            abpoa_generate_rc_msa(ab, abpt, seq_names, is_rc, tot_n, out_fp, msa_seq, msa_l);
+            abpoa_generate_rc_msa(ab, abpt, out_fp, msa_seq, msa_l);
         }
     }
-    free(is_rc);
     return 1;
 }
