@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "abpoa_align.h"
+#include "abpoa_seq.h"
 #include "simd_abpoa_align.h"
 #include "kdq.h"
 
@@ -55,16 +56,16 @@ void abpoa_set_graph_node(abpoa_graph_t *abg, int node_i) {
     abg->node[node_i].read_ids_n = 0;
 }
 
-void abpoa_free_node(abpoa_para_t *abpt, abpoa_node_t *node, int n) {
+void abpoa_free_node(abpoa_node_t *node, int n) {
     int i;
     for (i = 0; i < n; ++i) {
         if (node[i].in_edge_m > 0) free(node[i].in_id);
         if (node[i].out_edge_m > 0) { 
             free(node[i].out_id); free(node[i].out_weight);
-            if (abpt->use_read_ids) {
-                if (node[i].read_ids_n > 0)
-                    free(node[i].read_ids);
-            }
+            //if (abpt->use_read_ids) {
+            if (node[i].read_ids_n > 0)
+                free(node[i].read_ids);
+            //}
         }
         if (node[i].aligned_node_m > 0) free(node[i].aligned_node_id);
     }
@@ -108,12 +109,11 @@ abpoa_graph_t *abpoa_init_graph(void) {
     abg->is_topological_sorted = abg->is_called_cons = 0;
     abg->node_id_to_index = NULL; abg->index_to_node_id = NULL; abg->node_id_to_msa_rank = NULL;
     abg->node_id_to_max_pos_left = NULL; abg->node_id_to_max_pos_right = NULL; abg->node_id_to_max_remain = NULL;
-    abg->cal_R_time = 0;
     return abg;
 }
 
-void abpoa_free_graph(abpoa_graph_t *abg, abpoa_para_t *abpt) {
-    if (abg->node_m > 0) abpoa_free_node(abpt, abg->node, abg->node_m);
+void abpoa_free_graph(abpoa_graph_t *abg) {
+    if (abg->node_m > 0) abpoa_free_node(abg->node, abg->node_m);
 
     if (abg->node_n > 0) {
         free(abg->index_to_node_id);
@@ -130,12 +130,14 @@ void abpoa_free_graph(abpoa_graph_t *abg, abpoa_para_t *abpt) {
 abpoa_t *abpoa_init(void) {
     abpoa_t *ab = (abpoa_t*)_err_malloc(sizeof(abpoa_t));
     ab->abg = abpoa_init_graph();
+    ab->abs = abpoa_init_seq();
     ab->abm = abpoa_init_simd_matrix();
     return ab;
 }
 
-void abpoa_free(abpoa_t *ab, abpoa_para_t *abpt) {
-    abpoa_free_graph(ab->abg, abpt);
+void abpoa_free(abpoa_t *ab) {
+    abpoa_free_graph(ab->abg);
+    abpoa_free_seq(ab->abs);
     abpoa_free_simd_matrix(ab->abm);
     free(ab);
 }
@@ -187,7 +189,6 @@ next_out_node:;
 }
 
 void abpoa_BFS_set_node_remain(abpoa_graph_t *abg, int src_id, int sink_id) {
-    double real_start = realtime();
     int *id, cur_id, i, out_id, in_id;
 
     int *out_degree = (int*)_err_malloc(abg->node_n * sizeof(int));
@@ -220,8 +221,6 @@ void abpoa_BFS_set_node_remain(abpoa_graph_t *abg, int src_id, int sink_id) {
         }
         if (cur_id == src_id) {
             kdq_destroy_int(q); free(out_degree);
-            double real_end = realtime();
-            abg->cal_R_time += (real_end - real_start);
             return;
         }
         for (i = 0; i < abg->node[cur_id].in_edge_n; ++i) {
@@ -353,10 +352,10 @@ void abpoa_set_row_column_weight(abpoa_graph_t *abg, abpoa_para_t *abpt, int **r
     }
 }
 
-void abpoa_set_row_column_ids_weight(abpoa_graph_t *abg, abpoa_para_t *abpt, uint64_t ***read_ids, int **rc_weight, int **msa_node_id, int msa_l, int seq_n, int read_ids_n) {
+void abpoa_set_row_column_ids_weight(abpoa_graph_t *abg, abpoa_para_t *abpt, uint64_t ***read_ids, int **rc_weight, int **msa_node_id, int msa_l, int n_seq, int read_ids_n) {
     int i, j, k, rank, aligned_id;
     uint64_t b, one = 1, *whole_read_ids = (uint64_t*)_err_calloc(read_ids_n, sizeof(uint64_t));
-    for (i = 0; i < seq_n; ++i) {
+    for (i = 0; i < n_seq; ++i) {
         j = i / 64; b = i & 0x3f;
         whole_read_ids[j] |= (one << b);
     }
@@ -583,11 +582,11 @@ HB_CONS:
     free(score);
 }
 
-void abpoa_heaviest_column_consensus(abpoa_graph_t *abg, int **rc_weight, int **msa_node_id, int src_id, int sink_id, int msa_l, int seq_n) {
+void abpoa_heaviest_column_consensus(abpoa_graph_t *abg, int **rc_weight, int **msa_node_id, int src_id, int sink_id, int msa_l, int n_seq) {
     int i, j, w, max_base, max_w, gap_w;
     int last_id = src_id, cur_id;
     for (i = 0; i < msa_l; ++i) {
-        max_w = 0, max_base = 5, gap_w = seq_n;
+        max_w = 0, max_base = 5, gap_w = n_seq;
         for (j = 0; j < 4; ++j) {
             w = rc_weight[i][j];
             if (w > max_w) {
@@ -604,7 +603,7 @@ void abpoa_heaviest_column_consensus(abpoa_graph_t *abg, int **rc_weight, int **
     abg->node[last_id].max_out_id = sink_id;
 }
 
-void abpoa_heaviest_column(abpoa_graph_t *abg, abpoa_para_t *abpt, int src_id, int sink_id, int seq_n) {
+void abpoa_heaviest_column(abpoa_graph_t *abg, abpoa_para_t *abpt, int src_id, int sink_id, int n_seq) {
     abpoa_set_msa_rank(abg, src_id, sink_id);
 
     int i, msa_l = abg->node_id_to_msa_rank[sink_id] - 1;
@@ -612,12 +611,12 @@ void abpoa_heaviest_column(abpoa_graph_t *abg, abpoa_para_t *abpt, int src_id, i
     int **msa_node_id = (int**)_err_malloc(sizeof(int*) * msa_l);
     for (i = 0; i < msa_l; ++i) {
         rc_weight[i] = (int*)_err_calloc(5, sizeof(int)); // ACGT
-        rc_weight[i][4] = seq_n;
+        rc_weight[i][4] = n_seq;
         msa_node_id[i] = (int*)_err_calloc(5, sizeof(int));
     } 
     abpoa_set_row_column_weight(abg, abpt, rc_weight, msa_node_id);
 
-    abpoa_heaviest_column_consensus(abg, rc_weight, msa_node_id, src_id, sink_id, msa_l, seq_n);
+    abpoa_heaviest_column_consensus(abg, rc_weight, msa_node_id, src_id, sink_id, msa_l, n_seq);
     for (i = 0; i < msa_l; ++i) { 
         free(rc_weight[i]); free(msa_node_id[i]);
     } free(rc_weight); free(msa_node_id);
@@ -658,15 +657,15 @@ void add_het_read_ids(abpoa_para_t *abpt, int *init, uint64_t **het_read_ids, ui
     }
 }
 
-int set_clu_read_ids(int **clu_read_ids, int *clu_read_ids_n, int seq_n, double min_freq, uint64_t ***read_ids, uint8_t **het_cons_base, int *het_pos, int het_n) {
+int set_clu_read_ids(int **clu_read_ids, int *clu_read_ids_n, int n_seq, double min_freq, uint64_t ***read_ids, uint8_t **het_cons_base, int *het_pos, int het_n) {
     int i, j, seq_i, pos, clu_base;
-    int **diff = (int**)_err_malloc(sizeof(int*) * seq_n);
-    for (i = 0; i < seq_n; ++i) {
+    int **diff = (int**)_err_malloc(sizeof(int*) * n_seq);
+    for (i = 0; i < n_seq; ++i) {
         diff[i] = (int*)_err_malloc(2 *  sizeof(int));
         diff[i][0] = het_n, diff[i][1] = het_n;
     }
     uint64_t seq_b, one = 1; int seq_bit;
-    for (seq_i = 0; seq_i < seq_n; ++seq_i) {
+    for (seq_i = 0; seq_i < n_seq; ++seq_i) {
         seq_bit = seq_i / 64; seq_b = one << (seq_i & 0x3f); 
         for (i = 0; i < het_n; ++i) {
             pos = het_pos[i];
@@ -678,28 +677,28 @@ int set_clu_read_ids(int **clu_read_ids, int *clu_read_ids_n, int seq_n, double
             }
         }
     }
-    for (i = 0; i < seq_n; ++i) {
+    for (i = 0; i < n_seq; ++i) {
         if (diff[i][0] < diff[i][1]) {
             clu_read_ids[0][clu_read_ids_n[0]++] = i;
         } else if (diff[i][1] < diff[i][0]) {
             clu_read_ids[1][clu_read_ids_n[1]++] = i;
         }
     }
-    for (i = 0; i < seq_n; ++i) free(diff[i]); free(diff);
+    for (i = 0; i < n_seq; ++i) free(diff[i]); free(diff);
 
     int min_n = MIN_OF_TWO(clu_read_ids_n[0], clu_read_ids_n[1]);
-    int clu_n = (min_n >= (int)(seq_n * min_freq)) ? 2 : 1; 
+    int clu_n = (min_n >= (int)(n_seq * min_freq)) ? 2 : 1; 
     return clu_n;
 }
 
-int abpoa_diploid_ids(abpoa_para_t *abpt, uint64_t ***read_ids, int **rc_weight, int msa_l, int seq_n, double min_freq, int read_ids_n, int **clu_read_ids, int *clu_read_ids_n) {
+int abpoa_diploid_ids(abpoa_para_t *abpt, uint64_t ***read_ids, int **rc_weight, int msa_l, int n_seq, double min_freq, int read_ids_n, int **clu_read_ids, int *clu_read_ids_n) {
     int i, j;
     uint64_t **het_read_ids = (uint64_t**)_err_malloc(sizeof(uint64_t*) * 2 );
     for (i = 0; i < 2; ++i) het_read_ids[i] = (uint64_t*)_err_calloc(read_ids_n, sizeof(uint64_t));
     int *het_pos = (int*)_err_malloc(sizeof(int) * msa_l), het_n = 0;
     uint8_t **het_cons_base = (uint8_t**)_err_malloc(sizeof(uint8_t*) * msa_l);
     for (i = 0; i < msa_l; ++i) het_cons_base[i] = (uint8_t*)_err_malloc(sizeof(uint8_t) * 2);
-    int min_w = MAX_OF_TWO(1, seq_n * min_freq);
+    int min_w = MAX_OF_TWO(1, n_seq * min_freq);
     int init = 1, tmp, multip_i[2], multip_n, w;
     // collect het nodes
     for (i = 0; i < msa_l; ++i) {
@@ -730,7 +729,7 @@ int abpoa_diploid_ids(abpoa_para_t *abpt, uint64_t ***read_ids, int **rc_weight,
     else {
         // for each read, determine cluster based on diff with two cons-bases
         // return two group of read_ids
-        clu_n = set_clu_read_ids(clu_read_ids, clu_read_ids_n, seq_n, min_freq, read_ids, het_cons_base, het_pos, het_n);
+        clu_n = set_clu_read_ids(clu_read_ids, clu_read_ids_n, n_seq, min_freq, read_ids, het_cons_base, het_pos, het_n);
     }
     
     for (i = 0; i < msa_l; ++i) free(het_cons_base[i]);
@@ -739,10 +738,10 @@ int abpoa_diploid_ids(abpoa_para_t *abpt, uint64_t ***read_ids, int **rc_weight,
     return clu_n;
 }
 
-void abpoa_diploid_heaviest_column(abpoa_graph_t *abg, abpoa_para_t *abpt, int src_id, int sink_id, int seq_n, double min_freq, FILE *out_fp, uint8_t ***cons_seq, int **cons_l, int *cons_n) {
+void abpoa_diploid_heaviest_column(abpoa_graph_t *abg, abpoa_para_t *abpt, int src_id, int sink_id, int n_seq, double min_freq, FILE *out_fp, uint8_t ***cons_seq, int **cons_l, int *cons_n) {
     abpoa_set_msa_rank(abg, src_id, sink_id);
     int i, j, msa_l = abg->node_id_to_msa_rank[sink_id] - 1;
-    int read_ids_n = (seq_n-1)/64+1;
+    int read_ids_n = (n_seq-1)/64+1;
     uint64_t ***read_ids = (uint64_t***)_err_malloc(sizeof(uint64_t**) * msa_l);
     for (i = 0; i < msa_l; ++i) {
         read_ids[i] = (uint64_t**)_err_malloc(sizeof(uint64_t*) * 5);
@@ -752,17 +751,17 @@ void abpoa_diploid_heaviest_column(abpoa_graph_t *abg, abpoa_para_t *abpt, int s
     int **msa_node_id = (int**)_err_malloc(msa_l * sizeof(int*));
     for (i = 0; i < msa_l; ++i) {
         rc_weight[i] = (int*)_err_calloc(5, sizeof(int)); // ACGT
-        rc_weight[i][4] = seq_n;
+        rc_weight[i][4] = n_seq;
         msa_node_id[i] = (int*)_err_calloc(5, sizeof(int)); // ACGT
     } 
-    abpoa_set_row_column_ids_weight(abg, abpt, read_ids, rc_weight, msa_node_id, msa_l, seq_n, read_ids_n);
+    abpoa_set_row_column_ids_weight(abg, abpt, read_ids, rc_weight, msa_node_id, msa_l, n_seq, read_ids_n);
 
     int **clu_read_ids = (int**)_err_malloc(sizeof(int*) * 2), *clu_read_ids_n;
-    for (i = 0; i < 2; ++i) clu_read_ids[i] = (int*)_err_malloc(sizeof(int) * seq_n);
+    for (i = 0; i < 2; ++i) clu_read_ids[i] = (int*)_err_malloc(sizeof(int) * n_seq);
     clu_read_ids_n = (int*)_err_calloc(2, sizeof(int));
-    int clu_n = abpoa_diploid_ids(abpt, read_ids, rc_weight, msa_l, seq_n, min_freq, read_ids_n, clu_read_ids, clu_read_ids_n);
+    int clu_n = abpoa_diploid_ids(abpt, read_ids, rc_weight, msa_l, n_seq, min_freq, read_ids_n, clu_read_ids, clu_read_ids_n);
     if (clu_n == 1) {
-        abpoa_heaviest_column_consensus(abg, rc_weight, msa_node_id, src_id, sink_id, msa_l, seq_n);
+        abpoa_heaviest_column_consensus(abg, rc_weight, msa_node_id, src_id, sink_id, msa_l, n_seq);
         if (out_fp) output_consensus(abg, src_id, sink_id, out_fp);
         if (cons_n) {
             *cons_n = 1; abpoa_store_consensus(abg, src_id, sink_id, cons_seq, cons_l);
@@ -779,19 +778,20 @@ void abpoa_diploid_heaviest_column(abpoa_graph_t *abg, abpoa_para_t *abpt, int s
 }
 
 // should always topological sort first, then generate consensus
-int abpoa_generate_consensus(abpoa_t *ab, abpoa_para_t *abpt, int seq_n, FILE *out_fp, uint8_t ***cons_seq, int ***cons_cov, int **cons_l, int *cons_n) {
+int abpoa_generate_consensus(abpoa_t *ab, abpoa_para_t *abpt, FILE *out_fp, uint8_t ***cons_seq, int ***cons_cov, int **cons_l, int *cons_n) {
     abpoa_graph_t *abg = ab->abg;
     if (abg->node_n <= 2) return 0;
     int i, _cons_l = 0, *out_degree = (int*)_err_malloc(abg->node_n * sizeof(int));
+    int n_seq = ab->abs->n_seq;
     for (i = 0; i < abg->node_n; ++i) {
         out_degree[i] = abg->node[i].out_edge_n;
     }
 
     if (abpt->is_diploid) {
-        abpoa_diploid_heaviest_column(abg, abpt, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, seq_n, abpt->min_freq, out_fp, cons_seq, cons_l, cons_n);
+        abpoa_diploid_heaviest_column(abg, abpt, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, n_seq, abpt->min_freq, out_fp, cons_seq, cons_l, cons_n);
     } else {
         if (abpt->cons_agrm == ABPOA_HB) abpoa_heaviest_bundling(abg, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, out_degree, cons_cov);
-        else if (abpt->cons_agrm == ABPOA_HC) abpoa_heaviest_column(abg, abpt, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, seq_n);
+        else if (abpt->cons_agrm == ABPOA_HC) abpoa_heaviest_column(abg, abpt, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, n_seq);
         else if (abpt->cons_agrm == ABPOA_MF) abpoa_traverse_min_flow(abg, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, out_degree); 
         else err_fatal(__func__, "Unknown consensus calling algorithm: %d.", abpt->cons_agrm);
         if (out_fp) _cons_l = output_consensus(abg, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, out_fp);
@@ -813,7 +813,7 @@ void abpoa_set_msa_seq(abpoa_para_t *abpt, abpoa_node_t node, int rank, uint8_t
         while (num) {
             tmp = num & -num;
             read_id = ilog2_64(abpt, tmp);
-            // printf("%d -> %d\n", node.node_id, read_id);
+            // fprintf(stderr, "%d -> %d\n", node.node_id, read_id);
             msa_seq[b+read_id][rank-1] = base;
             num ^= tmp;
         }
@@ -821,22 +821,23 @@ void abpoa_set_msa_seq(abpoa_para_t *abpt, abpoa_node_t node, int rank, uint8_t
     }
 }
 
-void abpoa_generate_rc_msa(abpoa_t *ab, abpoa_para_t *abpt, char **read_names, uint8_t *is_rc, int seq_n, FILE *out_fp, uint8_t ***msa_seq, int *msa_l) {
+void abpoa_generate_rc_msa(abpoa_t *ab, abpoa_para_t *abpt, FILE *out_fp, uint8_t ***msa_seq, int *msa_l) {
     abpoa_graph_t *abg = ab->abg;
     if (abg->node_n <= 2) return;
     abpoa_set_msa_rank(abg, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID);
 
-    int i, j, k, aligned_id, rank;
-    uint8_t **_msa_seq = (uint8_t**)_err_malloc(seq_n * sizeof(uint8_t*));
+    abpoa_seq_t *abs = ab->abs;
+    int i, j, k, aligned_id, rank, n_seq = abs->n_seq;
+    uint8_t **_msa_seq = (uint8_t**)_err_malloc(n_seq * sizeof(uint8_t*));
     int _msa_l = abg->node_id_to_msa_rank[ABPOA_SINK_NODE_ID]-1;
 
-    for (i = 0; i < seq_n; ++i) {
+    for (i = 0; i < n_seq; ++i) {
         _msa_seq[i] = (uint8_t*)_err_malloc(_msa_l * sizeof(uint8_t));
         for (j = 0; j < _msa_l; ++j) 
             _msa_seq[i][j] = 5;
     }
 
-    if (out_fp && read_names == NULL) fprintf(out_fp, ">Multiple_sequence_alignment\n");
+    if (out_fp && abpt->out_msa_header == 0) fprintf(out_fp, ">Multiple_sequence_alignment\n");
     for (i = 2; i < abg->node_n; ++i) {
         // get msa rank
         rank = abpoa_graph_node_id_to_msa_rank(abg, i);
@@ -848,16 +849,16 @@ void abpoa_generate_rc_msa(abpoa_t *ab, abpoa_para_t *abpt, char **read_names, u
         abpoa_set_msa_seq(abpt, abg->node[i], rank, _msa_seq);
     }
     if (out_fp) {
-        for (i = 0; i < seq_n; ++i) {
-            if (read_names != NULL) {
-                if (is_rc[i]) fprintf(out_fp, ">%s_reverse_complement\n", read_names[i]);
-                else fprintf(out_fp, ">%s\n", read_names[i]);
+        for (i = 0; i < n_seq; ++i) {
+            if (abpt->out_msa_header && abs->name[i].l > 0) {
+                if (abs->is_rc[i]) fprintf(out_fp, ">%s_reverse_complement\n", abs->name[i].s);
+                else fprintf(out_fp, ">%s\n", abs->name[i].s);
             }
             for (j = 0; j < _msa_l; ++j) fprintf(out_fp, "%c", "ACGTN-"[_msa_seq[i][j]]);
             fprintf(out_fp, "\n");
         }
         if (abpt->out_cons) { // RC-MSA for consensus sequence
-            if (read_names) fprintf(out_fp, ">Consensus_sequence\n");
+            if (abpt->out_msa_header && abs->name[0].l > 0) fprintf(out_fp, ">Consensus_sequence\n");
 
             i = abg->node[ABPOA_SRC_NODE_ID].max_out_id;
             int last_rank = 1;
@@ -882,25 +883,26 @@ void abpoa_generate_rc_msa(abpoa_t *ab, abpoa_para_t *abpt, char **read_names, u
     if (msa_l) {
         *msa_l = _msa_l; *msa_seq = _msa_seq;
     } else {
-        for (i = 0; i < seq_n; ++i) free(_msa_seq[i]); free(_msa_seq);
+        for (i = 0; i < n_seq; ++i) free(_msa_seq[i]); free(_msa_seq);
     }
 }
 
-void abpoa_generate_gfa(abpoa_t *ab, abpoa_para_t *abpt, char **read_names, uint8_t *is_rc, int seq_n, FILE *out_fp) {
-    abpoa_graph_t *abg = ab->abg;
+void abpoa_generate_gfa(abpoa_t *ab, abpoa_para_t *abpt, FILE *out_fp) {
+    abpoa_seq_t *abs = ab->abs; abpoa_graph_t *abg = ab->abg;
     if (abg->node_n <= 2) return;
 
     // traverse graph 
     int *in_degree = (int*)_err_malloc(abg->node_n * sizeof(int));
-    int **read_paths = (int**)_err_malloc(seq_n * sizeof(int*)), *read_path_i = (int*)_err_calloc(seq_n, sizeof(int));
+    int n_seq = abs->n_seq;
+    int **read_paths = (int**)_err_malloc(n_seq * sizeof(int*)), *read_path_i = (int*)_err_calloc(n_seq, sizeof(int));
     int i, j, cur_id, pre_id, out_id, *id;
     for (i = 0; i < abg->node_n; ++i) in_degree[i] = abg->node[i].in_edge_n;
-    for (i = 0; i < seq_n; ++i) read_paths[i] = (int*)_err_malloc(abg->node_n * sizeof(int));
+    for (i = 0; i < n_seq; ++i) read_paths[i] = (int*)_err_malloc(abg->node_n * sizeof(int));
 
     // output comment and header
     int nl = 0;
     for (i = 2; i < abg->node_n; ++i) nl += abg->node[i].in_edge_n;
-    fprintf(out_fp, "H\tVN:Z:1.0\tNS:i:%d\tNL:i:%d\n", abg->node_n-2, nl - abg->node[ABPOA_SRC_NODE_ID].out_edge_n);
+    fprintf(out_fp, "H\tVN:Z:1.0\tNS:i:%d\tNL:i:%d\tNP:i:%d\n", abg->node_n-2, nl - abg->node[ABPOA_SRC_NODE_ID].out_edge_n, n_seq + abpt->out_cons);
 
     kdq_int_t *q = kdq_init_int();
 
@@ -944,10 +946,10 @@ void abpoa_generate_gfa(abpoa_t *ab, abpoa_para_t *abpt, char **read_names, uint
         }
     }
     // output read paths
-    for (i = 0; i < seq_n; ++i) {
-        if (read_names != NULL) fprintf(out_fp, "P\t%s\t", read_names[i]);
+    for (i = 0; i < n_seq; ++i) {
+        if (abs->name[i].l > 0) fprintf(out_fp, "P\t%s\t", abs->name[i].s);
         else fprintf(out_fp, "P\t%d\t", i+1);
-        if (is_rc[i]) {
+        if (abs->is_rc[i]) {
             for (j = read_path_i[i]-1; j >= 0; --j) {
                 fprintf(out_fp, "%d-", read_paths[i][j]);
                 if (j != 0) fprintf(out_fp, ",");
@@ -962,7 +964,7 @@ void abpoa_generate_gfa(abpoa_t *ab, abpoa_para_t *abpt, char **read_names, uint
         }
     }
     if (abpt->out_cons) {
-        abpoa_generate_consensus(ab, abpt, seq_n, NULL, NULL, NULL, NULL, NULL);
+        abpoa_generate_consensus(ab, abpt, NULL, NULL, NULL, NULL, NULL);
         int id = abg->node[ABPOA_SRC_NODE_ID].max_out_id;
         fprintf(out_fp, "P\tConsensus_sequence\t");
         while (1) {
@@ -976,16 +978,17 @@ void abpoa_generate_gfa(abpoa_t *ab, abpoa_para_t *abpt, char **read_names, uint
         }
     }
     free(in_degree);
-    for (i = 0; i < seq_n; ++i) free(read_paths[i]); 
+    for (i = 0; i < n_seq; ++i) free(read_paths[i]); 
     free(read_paths); free(read_path_i);
 }
 
 int abpoa_get_aligned_id(abpoa_graph_t *abg, int node_id, uint8_t base) {
-    int i;
+    int i, aln_id;
     abpoa_node_t *node = abg->node;
     for (i = 0; i < node[node_id].aligned_node_n; ++i) {
-        if (node[node[node_id].aligned_node_id[i]].base == base)
-            return abg->node[node_id].aligned_node_id[i];
+        aln_id = node[node_id].aligned_node_id[i];
+        if (node[aln_id].base == base)
+            return aln_id;
     }
     return -1;
 }
